@@ -1,16 +1,20 @@
 #include <R_ext/Rdynload.h>
 
+#include "thread_control.h"
 #include "sparseMatrix_utils.h"
 #include "SparseArray_class.h"
 #include "SVT_SparseArray_class.h"
-#include "SparseArray_subassignment.h"
+#include "SparseArray_dim_tuning.h"
+#include "SparseArray_aperm.h"
 #include "SparseArray_subsetting.h"
-#include "SparseArray_combine.h"
+#include "SparseArray_subassignment.h"
+#include "SparseArray_abind.h"
 #include "SparseArray_summarization.h"
 #include "SparseArray_Ops_methods.h"
 #include "SparseArray_Math_methods.h"
 #include "SparseArray_Complex_methods.h"
 #include "SparseMatrix_mult.h"
+#include "matrixStats_methods.h"
 #include "rowsum_methods.h"
 #include "randomSparseArray.h"
 #include "readSparseCSV.h"
@@ -20,6 +24,11 @@
 
 static const R_CallMethodDef callMethods[] = {
 
+/* thread_control.c */
+	CALLMETHOD_DEF(C_get_num_procs, 0),
+	CALLMETHOD_DEF(C_get_max_threads, 0),
+	CALLMETHOD_DEF(C_set_max_threads, 1),
+
 /* sparseMatrix_utils.c */
 	CALLMETHOD_DEF(C_colMins_dgCMatrix, 2),
 	CALLMETHOD_DEF(C_colMaxs_dgCMatrix, 2),
@@ -30,17 +39,24 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(C_coercion_can_introduce_zeros, 2),
 
 /* SVT_SparseArray_class.c */
-	CALLMETHOD_DEF(C_get_SVT_SparseArray_nzcount, 2),
 	CALLMETHOD_DEF(C_set_SVT_SparseArray_type, 4),
+	CALLMETHOD_DEF(C_nzcount_SVT_SparseArray, 2),
+	CALLMETHOD_DEF(C_nzwhich_SVT_SparseArray, 3),
 	CALLMETHOD_DEF(C_from_SVT_SparseArray_to_Rarray, 4),
 	CALLMETHOD_DEF(C_build_SVT_from_Rarray, 2),
 	CALLMETHOD_DEF(C_from_SVT_SparseMatrix_to_CsparseMatrix, 3),
 	CALLMETHOD_DEF(C_build_SVT_from_CsparseMatrix, 2),
 	CALLMETHOD_DEF(C_from_SVT_SparseArray_to_COO_SparseArray, 3),
-	CALLMETHOD_DEF(C_transpose_SVT_SparseMatrix, 3),
+
+/* SparseArray_dim_tuning.c */
+	CALLMETHOD_DEF(C_tune_SVT_dims, 4),
+
+/* SparseArray_aperm.c */
+	CALLMETHOD_DEF(C_transpose_2D_SVT, 3),
+	CALLMETHOD_DEF(C_aperm0_SVT, 4),
+	CALLMETHOD_DEF(C_aperm_SVT, 4),
 
 /* SparseArray_subsetting.c */
-	CALLMETHOD_DEF(C_drop_SVT_SparseArray_ineffective_dims, 4),
 	CALLMETHOD_DEF(C_subset_SVT_SparseArray, 4),
 
 /* SparseArray_subassignment.c */
@@ -50,13 +66,11 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(C_subassign_SVT_with_Rarray, 5),
 	CALLMETHOD_DEF(C_subassign_SVT_with_SVT, 7),
 
-/* SparseArray_combine.c */
+/* SparseArray_abind.c */
 	CALLMETHOD_DEF(C_abind_SVT_SparseArray_objects, 3),
 
 /* SparseArray_summarization.c */
-	CALLMETHOD_DEF(C_summarize_SVT_SparseArray, 6),
-	CALLMETHOD_DEF(C_count_SVT_SparseArray_NAs, 3),
-	CALLMETHOD_DEF(C_anyNA_SVT_SparseArray, 3),
+	CALLMETHOD_DEF(C_summarize_SVT, 6),
 
 /* SparseArray_Ops_methods.c */
 	CALLMETHOD_DEF(C_unary_minus_SVT, 3),
@@ -67,11 +81,10 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(C_Logic_SVT1_SVT2, 7),
 
 /* SparseArray_Math_methods.c */
-	CALLMETHOD_DEF(C_SVT_Math, 4),
-	CALLMETHOD_DEF(C_SVT_Math2, 5),
+	CALLMETHOD_DEF(C_Math_SVT, 5),
 
 /* SparseArray_Complex_methods.c */
-	CALLMETHOD_DEF(C_SVT_Complex, 4),
+	CALLMETHOD_DEF(C_Complex_SVT, 4),
 
 /* SparseMatrix_mult.c */
 	CALLMETHOD_DEF(C_crossprod2_SVT_mat, 7),
@@ -79,6 +92,9 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(C_crossprod2_SVT_SVT, 8),
 	CALLMETHOD_DEF(C_crossprod1_SVT, 5),
 
+/* matrixStats_methods.h.c */
+	CALLMETHOD_DEF(C_colStats_SVT, 8),
+
 /* rowsum_methods.c */
 	CALLMETHOD_DEF(C_rowsum_SVT, 6),
 	CALLMETHOD_DEF(C_rowsum_dgCMatrix, 4),
